searchNodes=[{"doc":"A task responsible for downloading the precompiled NIFs for a given module. This task must only be used by package creators who want to ship the precompiled NIFs. The goal is to download the precompiled packages and generate a checksum to check-in alongside the project in the the Hex repository. This is done by passing the --all flag. You can also use the --only-local flag to download only the precompiled package for use during development. You can use the --ignore-unavailable flag to ignore any NIFs that are not available. This is useful when you are developing a new NIF that does not support all platforms. This task also accept the --print flag to print the checksums.","ref":"Mix.Tasks.RustlerPrecompiled.Download.html","title":"Mix.Tasks.RustlerPrecompiled.Download","type":"task"},{"doc":"Download and use precompiled NIFs safely with checksums. Rustler Precompiled is a tool for library maintainers that rely on Rustler. It helps by removing the need to have the Rust compiler installed in the user's machine. Check the Precompilation Guide for details. Example defmodule MyNative do use RustlerPrecompiled , otp_app : :my_app , crate : &quot;my_app_nif&quot; , base_url : &quot;https://github.com/me/my_project/releases/download/v0.1.0&quot; , version : &quot;0.1.0&quot; end Options :otp_app - The OTP app name that the dynamic library will be loaded from. :crate - The name of Rust crate if different from the :otp_app . This is optional. :base_url - A valid URL that is used as base path for the NIF file. :version - The version of precompiled assets (it is part of the NIF filename). :force_build - Force the build with Rustler . This is false by default, but if your :version is a pre-release (like &quot;2.1.0-dev&quot;), this option will always be set true . You can also configure this option by setting an application env like this: config :rustler_precompiled , :force_build , your_otp_app : true It is important to add the &quot;:rustler&quot; package to your dependencies in order to force the build. To do that, just add it to your mix.exs file: { :rustler , &quot;&gt;= 0.0.0&quot; , optional : true } :targets - A list of targets supported by Rust for which precompiled assets are available. By default the following targets are configured: aarch64-apple-darwin aarch64-unknown-linux-gnu aarch64-unknown-linux-musl arm-unknown-linux-gnueabihf riscv64gc-unknown-linux-gnu x86_64-apple-darwin x86_64-pc-windows-gnu x86_64-pc-windows-msvc x86_64-unknown-linux-gnu x86_64-unknown-linux-musl :nif_versions - A list of OTP NIF versions for which precompiled assets are available. A NIF version is usually compatible with two OTP minor versions, and an older NIF is usually compatible with newer OTPs. The available versions are the following: 2.14 - for OTP 21. 2.15 - for OTP 22 and 23. 2.16 - for OTP 24 and 25. By default the following NIF versions are configured: 2.15 2.16 Check the compatibiliy table between Elixir and OTP in: https://hexdocs.pm/elixir/compatibility-and-deprecations.html#compatibility-between-elixir-and-erlang-otp In case &quot;force build&quot; is used, all options except :base_url , :version , :force_build , :nif_versions , and :targets are going to be passed down to Rustler . So if you need to configure the build, check the Rustler options. Environment variables This project reads some system environment variables. They are all optional, but they can change the behaviour of this library at compile time of your project. They are: HTTP_PROXY or http_proxy - Sets the HTTP proxy configuration. HTTPS_PROXY or https_proxy - Sets the HTTPS proxy configuration. MIX_XDG - If present, sets the OS as :linux for the :filename.basedir/3 when getting an user cache dir. TARGET_ARCH - The CPU target architecture. This is useful for when building your Nerves project, where your host CPU is different from your target CPU. Note that Nerves sets this value automatically when building your project. Examples: arm , aarch64 , x86_64 , riscv64 . TARGET_ABI - The target ABI (e.g., gnueabihf , musl ). This is set by Nerves as well. TARGET_VENDOR - The target vendor (e.g., unknown , apple , pc ). This is not set by Nerves. If any of the TARGET_ env vars is set, but TARGET_VENDOR is empty, then we change the target vendor to unknown that is the default value for Linux systems. TARGET_OS - The target operational system. This is always linux for Nerves. For more details about Nerves env vars, see https://hexdocs.pm/nerves/environment-variables.html","ref":"RustlerPrecompiled.html","title":"RustlerPrecompiled","type":"module"},{"doc":"Returns URLs for NIFs based on its module name. The module name is the one that defined the NIF and this information is stored in a metadata file.","ref":"RustlerPrecompiled.html#available_nif_urls/1","title":"RustlerPrecompiled.available_nif_urls/1","type":"function"},{"doc":"Returns the file URL to be downloaded for current target. It receives the NIF module.","ref":"RustlerPrecompiled.html#current_target_nif_url/1","title":"RustlerPrecompiled.current_target_nif_url/1","type":"function"},{"doc":"Returns the target triple for download or compile and load. This function is translating and adding more info to the system architecture returned by Elixir/Erlang to one used by Rust. The returned string has the following format: &quot;nif-NIF_VERSION-ARCHITECTURE-VENDOR-OS-ABI&quot; Examples iex&gt; RustlerPrecompiled . target ( ) { :ok , &quot;nif-2.16-x86_64-unknown-linux-gnu&quot; } iex&gt; RustlerPrecompiled . target ( ) { :ok , &quot;nif-2.15-aarch64-apple-darwin&quot; }","ref":"RustlerPrecompiled.html#target/3","title":"RustlerPrecompiled.target/3","type":"function"},{"doc":"Rustler provides an easy way to use safer NIFs in OTP applications. But in some environments it's harder to use the benefits of the tool because every user needs to install the Rust toolchain and compile the project, which can take several minutes in some cases. This changes with the help of the RustlerPrecompiled package. Now we can easily use precompiled Rustler NIFs from an external source. The precompilation happens in a CI server, always in a transparent way, and the Hex package published should always include a checksum file to ensure the NIFs stays the same, therefore avoiding supply chain attacks. In this guide I will show you how to prepare your project to use this feature.","ref":"precompilation_guide.html","title":"Precompilation guide","type":"extras"},{"doc":"Most of the work is done in the CI server. In this example we are going to use GitHub Actions. The GH Actions service has the benefit of hosting artifacts for releases and make them public available. Configure Github Actions In order for the workflow to succeed, read and write permissions will need to be enabled for the repository. Settings &gt; Actions &gt; General Workflow permissions Check the box &quot;Read and write permissions&quot; Configure Targets Usually we want to build for the most popular targets and the three last NIF versions. NIF versions are more stable than OTP versions because they only change after two major releases of OTP. For this guide our targets will be the following: OS: Linux, Windows, macOS Architectures: x86_64 , aarch64 (ARM 64 bits), arm NIF versions: 2.15 , 2.16 . In summary the build matrix looks like this: matrix: nif: [&quot;2.16&quot;, &quot;2.15&quot;] job: - { target: arm-unknown-linux-gnueabihf , os: ubuntu-20.04 , use-cross: true } - { target: aarch64-unknown-linux-gnu , os: ubuntu-20.04 , use-cross: true } - { target: aarch64-apple-darwin , os: macos-11 } - { target: x86_64-apple-darwin , os: macos-11 } - { target: x86_64-unknown-linux-gnu , os: ubuntu-20.04 } - { target: x86_64-unknown-linux-musl , os: ubuntu-20.04 , use-cross: true } - { target: x86_64-pc-windows-gnu , os: windows-2019 } - { target: x86_64-pc-windows-msvc , os: windows-2019 } A complete workflow example can be found in the rustler_precompilation_example project. That workflow is using a GitHub Action especially made for our goal: philss/rustler-precompiled-action . The GitHub Action will deal with the installation of cross and the build of the project, naming the files in the correct format. Some targets are only supported by later versions of cross . For those, you might want to install cross directly from GitHub. You can see an example in this pipeline .","ref":"precompilation_guide.html#prepare-for-the-build","title":"Precompilation guide - Prepare for the build","type":"extras"},{"doc":"In our build we are going to cross compile our crate project (the Rust code for our NIF) using a variety of targets, as we saw in the previous section. For this to work we need to guide the Rust compiler in some cases by providing additional configuration in the .cargo/config file of our project. Here is an example of that file: [target.'cfg(target_os = &quot;macos&quot;)'] rustflags = [ &quot;-C&quot;, &quot;link-arg=-undefined&quot;, &quot;-C&quot;, &quot;link-arg=dynamic_lookup&quot;, ] # See https://github.com/rust-lang/rust/issues/59302 [target.x86_64-unknown-linux-musl] rustflags = [ &quot;-C&quot;, &quot;target-feature=-crt-static&quot; ] # Provides a small build size, but takes more time to build. [profile.release] lto = true In addition to that, we also use a tool called cross that makes the build easier for some targets (the ones using use-cross: true in our example). We need to tell cross to read an environment variable from our &quot;host machine&quot;, because cross uses containers to build our software. So you need to create the file Cross.toml in the NIF directory with the following content: [build.env] passthrough = [ &quot;RUSTLER_NIF_VERSION&quot; ]","ref":"precompilation_guide.html#additional-configuration-before-build","title":"Precompilation guide - Additional configuration before build","type":"extras"},{"doc":"We need to tell RustlerPrecompiled where to find our NIF files, and we need to tell which version to use. defmodule RustlerPrecompilationExample.Native do version = Mix.Project . config ( ) [ :version ] use RustlerPrecompiled , otp_app : :rustler_precompilation_example , crate : &quot;example&quot; , base_url : &quot;https://github.com/philss/rustler_precompilation_example/releases/download/v \#{ version } &quot; , force_build : System . get_env ( &quot;RUSTLER_PRECOMPILATION_EXAMPLE_BUILD&quot; ) in [ &quot;1&quot; , &quot;true&quot; ] , version : version # When your NIF is loaded, it will override this function. def add ( _a , _b ) , do : :erlang . nif_error ( :nif_not_loaded ) end This example was extracted from the rustler_precompilation_example project. RustlerPrecompiled will try to figure out the target and download the correct file for us. This will happen in compile time only. Optionally it's possible to force the compilation by setting an env var, like the example suggests. It's also possible to force the build by using a pre release version, like 0.1.0-dev . The only requirement to force the build is to have Rustler declared as a dependency as well: {:rustler, &quot;&gt;= 0.0.0&quot;, optional: true} .","ref":"precompilation_guide.html#the-rustler-module","title":"Precompilation guide - The Rustler module","type":"extras"},{"doc":"Generating a checksum file In a scenario where you need to release a Hex package using precompiled NIFs, you first need to build the release in the CI, wait for all artifacts to be available and then generate the checksum file that is MANDATORY for your package to work. This checksum file is generated by running the following command after the build is complete: $ mix rustler_precompiled.download YourRustlerModule --all --print With the module I used for this guide, the command would be: $ mix rustler_precompiled.download RustlerPrecompilationExample.Native --all --print The file generated will be named checksum-Elixir.RustlerPrecompilationExample.Native.exs and it's extremely important that you include this file in your Hex package (by updating the files: field in your mix.exs ). Otherwise your package won't work . Your files: key at your package configuration will look like this: defp package do [ files : [ &quot;lib&quot; , &quot;native/example/.cargo&quot; , &quot;native/example/src&quot; , &quot;native/example/Cargo*&quot; , &quot;checksum-*.exs&quot; , &quot;mix.exs&quot; ] , # ... ] end Note: you don't need to track the checksum file in your version control system (git or other). For an example, refer to the mix.exs file of the rustler precompilation example or elixir-nx's explorer library. Tip: use the mix hex.build --unpack command to confirm which files are being included (and if the package looks good before publishing). Recommended flow To recap, the suggested flow is the following: release a new tag push the code to your repository with the new tag: git push origin main --tags wait for all NIFs to be built run the mix rustler_precompiled.download task (with the flag --all ) release the package to Hex.pm (make sure your release includes the correct files).","ref":"precompilation_guide.html#the-release-flow","title":"Precompilation guide - The release flow","type":"extras"},{"doc":"The ability to use precompiled NIFs written in Rust can increase the adoption of some packages, because people won't need to have Rust installed. But this comes with some drawbacks and more responsibilities to the maintainers, so use this feature carefully.","ref":"precompilation_guide.html#conclusion","title":"Precompilation guide - Conclusion","type":"extras"},{"doc":"All notable changes to this project will be documented in this file. The format is based on Keep a Changelog , and this project adheres to Semantic Versioning .","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"","ref":"changelog.html#unreleased","title":"Changelog - Unreleased","type":"extras"},{"doc":"Changed Depend on :ssl instead of :public_key application. Since :public_key is started with :ssl , this shouldn't break. This change is needed in order to support the upcoming Elixir 1.15.","ref":"changelog.html#0-6-1-2023-02-16","title":"Changelog - 0.6.1 - 2023-02-16","type":"extras"},{"doc":"Added Add support for configuring the NIF versions that the project supports. This can be done with the :nif_versions config. Add support for castore version ~&gt; 1.0 . Changed Add aarch64-unknown-linux-musl and riscv64gc-unknown-linux-gnu as default targets. The first one is common for people running Linux containers that were built with Musl on Apple computers. The second one is becoming popular for tiny computers, normally running Nerves. The adoption of these targets can increase a little bit the compilation time, but can affect a great number of users. For package maintainers: please remember to add these targets to your CI workflow. See an example workflow at: https://github.com/philss/rustler_precompilation_example/blob/main/.github/workflows/release.yml Change the depth of SSL peer verification to &quot;3&quot;. This should be more compatible with servers. Remove version &quot;2.14&quot; from the default NIF versions. Like the change of default targets, this should only have effect in the moment of release of a new package version. Remember to update your workflow file.","ref":"changelog.html#0-6-0-2023-01-27","title":"Changelog - 0.6.0 - 2023-01-27","type":"extras"},{"doc":"Fixed Add support for Suse Linux targets. This is a fix to the plataform resolution. Thanks @fabriziosestito . Fix validation of HTTP proxy. This makes the validation similar to the HTTPS proxy. Thanks @w0rd-driven . Map riscv64 to riscv64gc to match Rust naming. Thanks @fhunleth .","ref":"changelog.html#0-5-5-2022-12-10","title":"Changelog - 0.5.5 - 2022-12-10","type":"extras"},{"doc":"Fixed Fix building metadata when &quot;force build&quot; is enabled and the target is not available.","ref":"changelog.html#0-5-4-2022-11-05","title":"Changelog - 0.5.4 - 2022-11-05","type":"extras"},{"doc":"Fixed Always write the metadata file in compilation time, so mix tasks can work smoothly.","ref":"changelog.html#0-5-3-2022-10-19","title":"Changelog - 0.5.3 - 2022-10-19","type":"extras"},{"doc":"Fixed Fix the target/0 function to use default targets a default argument. This makes the example in the docs work again. Thanks @jackalcooper . Only use proxy if it is valid. Thanks @josevalim . Fix the support for PCs running RedHat Linux. Thanks @Benjamin-Philip . Improve some points in the docs. Thanks @whatyouhide and @fabriziosestito .","ref":"changelog.html#0-5-2-2022-10-03","title":"Changelog - 0.5.2 - 2022-10-03","type":"extras"},{"doc":"Fixed Fix available targets naming to include the NIF version in the name. It was removed accidentally. Thanks @adriankumpf .","ref":"changelog.html#0-5-1-2022-05-24","title":"Changelog - 0.5.1 - 2022-05-24","type":"extras"},{"doc":"Added Now it's possible to configure the targets list, based in the Rust's Plataform Support list. You can run rustc --print target-list to get the full list. Thanks @adriankumpf . Changed The precompilation guide was improved with instructions and suggestions for the files key at the project config. Thanks @nbw . Now we raise with a different error if the NIF artifact cannot be written when downloading to create the checksum file.","ref":"changelog.html#0-5-0-2022-05-24","title":"Changelog - 0.5.0 - 2022-05-24","type":"extras"},{"doc":"Fixed Fix __using__ macro for when Rustler is not loaded.","ref":"changelog.html#0-4-1-2022-04-28","title":"Changelog - 0.4.1 - 2022-04-28","type":"extras"},{"doc":"Changed Make Rustler an optional dependency. This makes installation faster for most of the users.","ref":"changelog.html#0-4-0-2022-04-28","title":"Changelog - 0.4.0 - 2022-04-28","type":"extras"},{"doc":"Added Add the possibility to skip the download of unavailable NIFs when generating the checksum file - thanks @fahchen","ref":"changelog.html#0-3-0-2022-03-26","title":"Changelog - 0.3.0 - 2022-03-26","type":"extras"},{"doc":"Fixed Fix validation of URL in order to be compatible with Elixir ~&gt; 1.11. The previous implementation was restricted to Elixir ~&gt; 1.13. Added Add :force_build option that fallback to Rustler . It passes all options except the ones used by RustlerPrecompiled down to Rustler . This option will be by default false , but if the project is using a pre-release, then it will always be set to true . With this change the project starts depending on Rustler. Changed Relax dependencies to the minor versions.","ref":"changelog.html#0-2-0-2022-02-18","title":"Changelog - 0.2.0 - 2022-02-18","type":"extras"},{"doc":"Added Add basic features to download and use the precompiled NIFs in a safe way.","ref":"changelog.html#0-1-0-2022-02-16","title":"Changelog - 0.1.0 - 2022-02-16","type":"extras"}]